@@ -52,7 +52,6 @@
 #include <QtCore/QRegExp>
 #include <QFont>
 #include <kcrash.h>
-#include <klibrary.h>
 #include <kconfig.h>
 #include <klocalizedstring.h>
 #include <QDebug>
@@ -88,10 +87,10 @@
 #ifdef Q_OS_UNIX
 //TODO: make sure what libraries we want here...
 static const char *extra_libs[] = {
-    "libKIOCore.so.5",
-    "libKParts.so.5",
+    "libKF5KIOCore.so.5",
+    "libKF5Parts.so.5",
 //#ifdef __KDE_HAVE_GCC_VISIBILITY // Removed for KF5, we'll see.
-    "libPlasma.so.5"
+    "libKF5Plasma.so.5"
 //#endif
 };
 #endif
@@ -472,34 +471,34 @@ static pid_t launch(int argc, const char *_name, const char *args,
 {
     QString lib;
     QByteArray name;
-    QByteArray exec;
     QString libpath;
     QByteArray execpath;
+    bool libpath_relative = false;
 
     if (_name[0] != '/') {
         name = _name;
         lib = QFile::decodeName(name);
-        exec = name;
-        KLibrary klib(QLatin1String("libkdeinit5_") + lib);
-        libpath = klib.fileName();
-        if (libpath.isEmpty()) {
-            KLibrary klib(lib);
-            libpath = klib.fileName(); // this is really just a way to call findLibraryInternal
-        }
-        execpath = execpath_avoid_loops(exec, envc, envs, avoid_loops);
-        if (libpath.isEmpty() && execpath.isEmpty()) {
-            fprintf(stderr, "Didn't find \"%s\", neither as an executable nor as a plugin. Please check $PATH and $QT_PLUGIN_PATH.\n", name.constData());
-        }
+        libpath = QLatin1String("libkdeinit5_") + lib;
+        libpath_relative = true;
+        execpath = execpath_avoid_loops(name, envc, envs, avoid_loops);
     } else {
         name = _name;
         lib = QFile::decodeName(name);
         name = name.mid(name.lastIndexOf('/') + 1);
-        exec = _name;
+
+        // FIXME: this .so extension stuff is very Linux-specific
         if (lib.endsWith(QLatin1String(".so"))) {
             libpath = lib;
         } else {
-            // try to match an absolute path to an executable binary (either in bin/ or in libexec/)
-            // to a kdeinit module in the same prefix
+            execpath = _name;
+
+            // Try to match an absolute path to an executable binary (either in
+            // bin/ or in libexec/) to a kdeinit module in the same prefix.
+            //
+            // Note that these *_INSTALL_DIR values should normally relative to
+            // the install prefix, although this may not be the case if the user
+            // has overridden them, and so this search is inherently fragile in
+            // the face of unusual installation layouts.
             if (lib.contains(QLatin1String(LIBEXEC_INSTALL_DIR))) {
                 libpath = QString(lib).replace(QLatin1String(LIBEXEC_INSTALL_DIR),
                                                QLatin1String(LIB_INSTALL_DIR "/libkdeinit5_")) + QLatin1String(".so");
@@ -511,7 +510,6 @@ static pid_t launch(int argc, const char *_name, const char *args,
             if (!QFile::exists(libpath)) {
                 libpath.clear();
             }
-            execpath = exec;
         }
     }
 #ifndef NDEBUG
@@ -630,12 +628,12 @@ static pid_t launch(int argc, const char *_name, const char *args,
             /* set the process name, so that killall works like intended */
             r = prctl(PR_SET_NAME, (unsigned long) name.data(), 0, 0, 0);
             if (r == 0) {
-                proctitle_set("%s [kdeinit]%s", name.data(), procTitle.data() ? procTitle.data() : "");
+                proctitle_set("-%s [kdeinit5]%s", name.data(), procTitle.data() ? procTitle.data() : "");
             } else {
-                proctitle_set("kdeinit5: %s%s", name.data(), procTitle.data() ? procTitle.data() : "");
+                proctitle_set("%s%s", name.data(), procTitle.data() ? procTitle.data() : "");
             }
 #else
-            proctitle_set("kdeinit5: %s%s", name.data(), procTitle.data() ? procTitle.data() : "");
+            proctitle_set("%s%s", name.data(), procTitle.data() ? procTitle.data() : "");
 #endif
 #endif
         }
@@ -652,7 +650,20 @@ static pid_t launch(int argc, const char *_name, const char *args,
         QLibrary l(libpath);
 
         if (!libpath.isEmpty()) {
-            if (!l.load() || !l.isLoaded()) {
+            if (!l.load()) {
+                if (libpath_relative) {
+                    // NB: Because Qt makes the actual dlopen() call, the
+                    //     RUNPATH of kdeinit is *not* respected - see
+                    //     https://sourceware.org/bugzilla/show_bug.cgi?id=13945
+                    //     - so we try hacking it in ourselves
+                    QString install_lib_dir = QFile::decodeName(
+                            CMAKE_INSTALL_PREFIX "/" LIB_INSTALL_DIR "/");
+                    libpath = install_lib_dir + libpath;
+                    l.setFileName(libpath);
+                    l.load();
+                }
+            }
+            if (!l.isLoaded()) {
                 QString ltdlError(l.errorString());
                 if (execpath.isEmpty()) {
                     // Error
@@ -660,7 +671,7 @@ static pid_t launch(int argc, const char *_name, const char *args,
                     exitWithErrorMsg(errorMsg);
                 } else {
                     // Print warning
-                    fprintf(stderr, "Could not open library %s: %s\n", qPrintable(lib),
+                    fprintf(stderr, "Could not open %s using a library: %s\n", qPrintable(lib),
                             qPrintable(ltdlError));
                 }
             }
@@ -883,17 +894,6 @@ static void init_kdeinit_socket()
                 exit(255);
             }
         }
-#if 0 // obsolete in kde4. Should we check writing to another file instead?
-        path = qgetenv("ICEAUTHORITY");
-        if (path.isEmpty()) {
-            path = home_dir;
-            path += "/.ICEauthority";
-        }
-        if (access(path.data(), R_OK | W_OK) && (errno != ENOENT)) {
-            fprintf(stderr, "kdeinit5: Aborting. No write access to '%s'.\n", path.data());
-            exit(255);
-        }
-#endif
     }
 
     /** Test if socket file is already present
@@ -1380,62 +1380,9 @@ static void handle_requests(pid_t waitForPid)
     }
 }
 
-static void kdeinit_library_path()
+static void generate_socket_name()
 {
 
-    const QStringList ltdl_library_path =
-        QFile::decodeName(qgetenv("LTDL_LIBRARY_PATH")).split(QLatin1Char(':'), QString::SkipEmptyParts);
-#ifdef Q_OS_DARWIN
-    const QByteArray ldlibpath = qgetenv("DYLD_LIBRARY_PATH");
-#else
-    const QByteArray ldlibpath = qgetenv("LD_LIBRARY_PATH");
-#endif
-    const QStringList ld_library_path =
-        QFile::decodeName(ldlibpath).split(QLatin1Char(':'), QString::SkipEmptyParts);
-
-#if 0 // unused
-
-    QByteArray extra_path;
-    const QStringList candidates = s_instance->dirs()->resourceDirs("lib");
-    for (QStringList::ConstIterator it = candidates.begin();
-            it != candidates.end();
-            ++it) {
-        QString d = *it;
-        if (ltdl_library_path.contains(d)) {
-            continue;
-        }
-        if (ld_library_path.contains(d)) {
-            continue;
-        }
-        if (d[d.length() - 1] == QLatin1Char('/')) {
-            d.truncate(d.length() - 1);
-            if (ltdl_library_path.contains(d)) {
-                continue;
-            }
-            if (ld_library_path.contains(d)) {
-                continue;
-            }
-        }
-        if ((d == QLatin1String("/lib")) || (d == QLatin1String("/usr/lib"))) {
-            continue;
-        }
-
-        QByteArray dir = QFile::encodeName(d);
-
-        if (access(dir, R_OK)) {
-            continue;
-        }
-
-        if (!extra_path.isEmpty()) {
-            extra_path += ':';
-        }
-        extra_path += dir;
-    }
-#endif
-
-//   if (!extra_path.isEmpty())
-//      lt_dlsetsearchpath(extra_path.data());
-
     QByteArray display = qgetenv(displayEnvVarName().constData());
     if (display.isEmpty()) {
 #if HAVE_X11 // qt5: see displayEnvVarName()
@@ -1677,7 +1624,7 @@ extern "C" {
 
 }
 
-int main(int argc, char **argv, char **envp)
+int main(int argc, char **argv)
 {
 #ifndef _WIN32_WCE
     setlocale(LC_ALL, "");
@@ -1687,10 +1634,7 @@ int main(int argc, char **argv, char **envp)
     pid_t pid;
     bool do_fork = true;
     int launch_klauncher = 1;
-    // KDE5 TODO: make this 0 by default, so that a random kde app doesn't start kded5 upfront,
-    // but on demand when a service needs it. In startkde, however, use +kded5 or --kded.
-    // (+kded5 might not work because it doesn't fork on startup anymore)
-    int launch_kded = 1;
+    int launch_kded = 0;
     int keep_running = 1;
     d.suicide = false;
 
@@ -1701,16 +1645,17 @@ int main(int argc, char **argv, char **envp)
         if (strcmp(safe_argv[i], "--no-klauncher") == 0) {
             launch_klauncher = 0;
         }
-        if (strcmp(safe_argv[i], "--no-kded") == 0) {
-            launch_kded = 0;
+        if (strcmp(safe_argv[i], "--kded") == 0) {
+            launch_kded = 1;
         }
-#ifdef Q_OS_MAC
-        // make it nofork to match KUniqueApplication, technically command-line incompatible
-        if (strcmp(safe_argv[i], "--nofork") == 0)
-#else
-        if (strcmp(safe_argv[i], "--no-fork") == 0)
-#endif
+        // allow both nofork and no-fork for compatibility with
+        // old versions (both of this and of KUniqueApplication)
+        if (strcmp(safe_argv[i], "--nofork") == 0) {
             do_fork = false;
+        }
+        if (strcmp(safe_argv[i], "--no-fork") == 0) {
+            do_fork = false;
+        }
         if (strcmp(safe_argv[i], "--suicide") == 0) {
             d.suicide = true;
         }
@@ -1729,13 +1674,9 @@ int main(int argc, char **argv, char **envp)
 #endif
         if (strcmp(safe_argv[i], "--help") == 0) {
             printf("Usage: kdeinit5 [options]\n");
-#ifdef Q_OS_MAC
-            printf("    --nofork          Do not fork\n");
-#else
             printf("    --no-fork         Do not fork\n");
-#endif
             // printf("    --no-klauncher    Do not start klauncher\n");
-            printf("    --no-kded         Do not start kded\n");
+            printf("    --kded            Start kded\n");
             printf("    --suicide         Terminate when no KDE applications are left running\n");
             printf("    --version         Show version information\n");
             // printf("    --exit            Terminate when kded has run\n");
@@ -1786,10 +1727,9 @@ int main(int argc, char **argv, char **envp)
 
     /** Prepare to change process name **/
 #ifndef SKIP_PROCTITLE
-    proctitle_init(argc, argv, envp);
+    proctitle_init(argc, argv);
 #endif
 
-    kdeinit_library_path();
     // don't change envvars before proctitle_init()
     unsetenv("LD_BIND_NOW");
     unsetenv("DYLD_BIND_AT_LAUNCH");
@@ -1808,6 +1748,7 @@ int main(int argc, char **argv, char **envp)
     setupX();
 #endif
 
+    generate_socket_name();
     if (keep_running) {
         /*
          * Create ~/.kde/tmp-<hostname>/kdeinit5-<display> socket for incoming wrapper
@@ -1894,7 +1835,7 @@ int main(int argc, char **argv, char **envp)
     free(safe_argv);
 
 #ifndef SKIP_PROCTITLE
-    proctitle_set("kdeinit5 Running...");
+    proctitle_set("Running...");
 #endif
 
     if (!keep_running) {
